@@ -3,26 +3,55 @@ require "uri"
 
 module Bundler
   class Settings
-    BOOL_KEYS = %w(frozen cache_all no_prune disable_local_branch_check disable_shared_gems ignore_messages gem.mit gem.coc silence_root_warning no_install).freeze
-    NUMBER_KEYS = %w(retry timeout redirect ssl_verify_mode).freeze
-    DEFAULT_CONFIG = { :retry => 3, :timeout => 10, :redirect => 5 }.freeze
+    BOOL_KEYS = %w(
+      allow_offline_install
+      cache_all
+      disable_exec_load
+      disable_local_branch_check
+      disable_shared_gems
+      frozen
+      gem.coc
+      gem.mit
+      ignore_messages
+      major_deprecations
+      no_install
+      no_prune
+      only_update_to_newer_versions
+      plugins
+      silence_root_warning
+    ).freeze
+
+    NUMBER_KEYS = %w(
+      redirect
+      retry
+      ssl_verify_mode
+      timeout
+    ).freeze
+
+    DEFAULT_CONFIG = {
+      :redirect => 5,
+      :retry => 3,
+      :timeout => 10,
+    }.freeze
+
+    attr_accessor :cli_flags_given
 
     def initialize(root = nil)
-      @root          = root
-      @local_config  = load_config(local_config_file)
-      @global_config = load_config(global_config_file)
+      @root            = root
+      @local_config    = load_config(local_config_file)
+      @global_config   = load_config(global_config_file)
+      @cli_flags_given = false
     end
 
     def [](name)
       key = key_for(name)
       value = (@local_config[key] || ENV[key] || @global_config[key] || DEFAULT_CONFIG[name])
 
-      case
-      when value.nil?
+      if value.nil?
         nil
-      when is_bool(name) || value == "false"
+      elsif is_bool(name) || value == "false"
         to_bool(value)
-      when is_num(name)
+      elsif is_num(name)
         value.to_i
       else
         value
@@ -30,6 +59,19 @@ module Bundler
     end
 
     def []=(key, value)
+      if cli_flags_given
+        command = if value.nil?
+          "bundle config --delete #{key}"
+        else
+          "bundle config #{key} #{Array(value).join(":")}"
+        end
+
+        Bundler::SharedHelpers.major_deprecation \
+          "flags passed to commands " \
+          "will no longer be automatically remembered. Instead please set flags " \
+          "you want remembered between commands using `bundle config " \
+          "<setting name> <setting value>`, i.e. `#{command}`"
+      end
       local_config_file || raise(GemfileNotFound, "Could not locate Gemfile")
       set_key(key, value, @local_config, local_config_file)
     end
@@ -201,8 +243,8 @@ module Bundler
         hash.delete(key) if value.nil?
         SharedHelpers.filesystem_access(file) do |p|
           FileUtils.mkdir_p(p.dirname)
-          require "bundler/psyched_yaml"
-          File.open(p, "w") {|f| f.puts YAML.dump(hash) }
+          require "bundler/yaml_serializer"
+          p.open("w") {|f| f.write(YAMLSerializer.dump(hash)) }
         end
       end
 
@@ -221,23 +263,28 @@ module Bundler
       Pathname.new(@root).join("config") if @root
     end
 
-    def load_config(config_file)
-      SharedHelpers.filesystem_access(config_file, :read) do
-        valid_file = config_file && config_file.exist? && !config_file.size.zero?
-        return {} if ignore_config? || !valid_file
-        config_regex = /^(BUNDLE_.+): (['"]?)(.*(?:\n(?!BUNDLE).+)?)\2$/
-        config_pairs = config_file.read.scan(config_regex).map do |m|
-          key, _, value = m
-          [convert_to_backward_compatible_key(key), value.gsub(/\s+/, " ").tr('"', "'")]
-        end
-        Hash[config_pairs]
-      end
-    end
+    CONFIG_REGEX = %r{ # rubocop:disable Style/RegexpLiteral
+      ^
+      (BUNDLE_.+):\s # the key
+      (?: !\s)? # optional exclamation mark found with ruby 1.9.3
+      (['"]?) # optional opening quote
+      (.* # contents of the value
+        (?: # optionally, up until the next key
+          (\n(?!BUNDLE).+)*
+        )
+      )
+      \2 # matching closing quote
+      $
+    }xo
 
-    def convert_to_backward_compatible_key(key)
-      key = "#{key}/" if key =~ /https?:/i && key !~ %r{/\Z}
-      key = key.gsub(".", "__") if key.include?(".")
-      key
+    def load_config(config_file)
+      return unless config_file
+      SharedHelpers.filesystem_access(config_file, :read) do |file|
+        valid_file = file.exist? && !file.size.zero?
+        return {} if ignore_config? || !valid_file
+        require "bundler/yaml_serializer"
+        YAMLSerializer.load file.read
+      end
     end
 
     # TODO: duplicates Rubygems#normalize_uri
@@ -247,7 +294,7 @@ module Bundler
       uri = "#{uri}/" unless uri =~ %r{/\Z}
       uri = URI(uri)
       unless uri.absolute?
-        raise ArgumentError, "Gem sources must be absolute. You provided '#{uri}'."
+        raise ArgumentError, format("Gem sources must be absolute. You provided '%s'.", uri)
       end
       uri
     end
