@@ -2,6 +2,10 @@
 module Bundler
   module UI
     class Silent
+      def add_color(string, color)
+        string
+      end
+
       def info(message, newline = nil)
       end
 
